##
## Copyright 2016-2020 Intel Corporation.
##
## This software and the related documents are Intel copyrighted  materials,  and
## your use of  them is  governed by the  express license  under which  they were
## provided to you (License).  Unless the License provides otherwise, you may not
## use, modify, copy, publish, distribute,  disclose or transmit this software or
## the related documents without Intel's prior written permission.
##
## This software and the related documents  are provided as  is,  with no express
## or implied  warranties,  other  than those  that are  expressly stated  in the
## License.
##

#
# Base commands
#
ARFLAGS ?= rvcs

#
# System configuration detection
#

SYSTEM_SFX := $(shell uname)
ifeq ($(SYSTEM_SFX),Darwin)
	G_SYSTEM := macOS
else
	G_SYSTEM := Linux
endif

ifndef $(ARCH)
	ARCH_SFX := $(shell uname -m)
	ifeq ($(ARCH_SFX),x86_64)
		ARCH := intel64
	else
		ARCH := ia32
	endif
endif

ifeq ($(ARCH),ia32)
	G_ARCH := ia32
endif
ifeq ($(ARCH),intel64)
	G_ARCH := intel64
endif
ifeq ($(G_ARCH),)
$(error ARCH is invalid)
endif

#
# Base build configuration
#
ifndef $(CONF)
	ifneq ($(TBBROOT),)
		CONF := release_tbb
	else
		CONF := release_omp
	endif
endif

ifeq ($(CONF),debug_omp)
	ifeq ($(DPCPP_EN),1)
		G_DPCPP_EN := 1
	endif
	G_CONF      := debug
	G_USETBB    := 0
	G_USEOMP    := 1
endif
ifeq ($(CONF),debug_tbb)
	G_CONF      := debug
	G_USETBB    := 1
	G_USEOMP    := 0
endif
ifeq ($(CONF),release_omp)
	ifeq ($(DPCPP_EN),1)
		G_DPCPP_EN := 1
	endif
	G_CONF      := release
	G_USETBB    := 0
	G_USEOMP    := 1
endif
ifeq ($(CONF),release_tbb)
	G_CONF      := release
	G_USETBB    := 1
	G_USEOMP    := 0
endif
ifeq ($(G_CONF),)
$(error CONF is invalid)
endif

ifeq ($(G_CONF),debug)
	G_CFLAGS   += -g -D_DEBUG
	G_CXXFLAGS += -g -D_DEBUG
endif
G_CFLAGS   += -Wformat -Wformat-security -std=c99 -pedantic
G_CXXFLAGS += -Wformat -Wformat-security

ifeq ($(G_SYSTEM),macOS)
	G_IPP_PATH_SUFFIX ?= lib
	G_TBB_PATH_SUFFIX ?= lib
else
ifeq ($(ARCH),intel64)
	G_IPP_PATH_SUFFIX ?= lib/intel64
	G_TBB_PATH_SUFFIX ?= lib/intel64/gcc4.4
endif
ifeq ($(ARCH),ia32)
	G_IPP_PATH_SUFFIX ?= lib/ia32
	G_TBB_PATH_SUFFIX ?= lib/ia32/gcc4.4
endif
endif

#
# Paths configuration
#
G_MAKE_ROOT      := $(shell pwd)/
G_BUILD_ROOT     := $(G_MAKE_ROOT)/_build/
$(shell mkdir -p $(G_BUILD_ROOT))

ifeq ($(G_USETBB),1)
	G_CONF_DIR := $(G_CONF)_tbb
else
	ifeq ($(G_USEOMP),1)
		G_CONF_DIR := $(G_CONF)_omp
	else
		$(error CONF is invalid)
	endif
endif

# Requred compilation flags:
# G_LIB_CFLAGS   - Library C flags
# G_LIB_LDFLAGS  - Library linker flags
# G_EXE_CFLAGS   - Executable C flags
# G_EXE_LDFLAGS  - Executable linker flags
G_LIB_CFLAGS   := -O3 -fPIC -D_FORTIFY_SOURCE=2

ifneq ($(DPCPP_EN),1)
	G_LIB_CFLAGS += -ffreestanding
endif

ifeq ($(G_SYSTEM),macOS)
	G_LIB_LDFLAGS  := 
else
	G_LIB_LDFLAGS  := -z noexecstack
endif
ifeq ($(G_SYSTEM),macOS)
	G_EXE_LDFLAGS  := -Wl,-pie
else
	G_EXE_LDFLAGS  := -pie -z noexecstack -z relro -z now
	G_EXE_CFLAGS   += -O3 -fPIC -D_FORTIFY_SOURCE=2
endif

G_EXE_LDFLAGS += -lpthread

ifeq ($(G_FSPS_SUPPORTED),1)
	G_LIB_CFLAGS += -fstack-protector-strong
	G_EXE_CFLAGS += -fstack-protector-strong
else
	G_LIB_CFLAGS += -fstack-protector
	G_EXE_CFLAGS += -fstack-protector
endif

ifneq ($(G_SYSTEM),macOS)
	G_LIB_CFLAGS += -D_XOPEN_SOURCE=500
	G_LIB_LDFLAGS += -Wl,-z,relro -Wl,-z,now -nostdlib
	G_EXE_CFLAGS += -D_XOPEN_SOURCE=500
endif

ifeq ($(G_USETBB),1)
	G_LIB_CFLAGS  += -DUSE_TBB -I$(TBBROOT)/include
	G_LIB_LDFLAGS += -L$(TBBROOT)/$(G_TBB_PATH_SUFFIX) -ltbb
	G_EXE_LDFLAGS += -L$(TBBROOT)/$(G_TBB_PATH_SUFFIX) -ltbb
	G_EXAMPLE_CFLAGS := $(G_EXE_CFLAGS) -DUSE_TBB -I$(TBBROOT)/include
endif
ifeq ($(G_USEOMP), 1)
	G_LIB_CFLAGS += -DUSE_OMP -fopenmp
	G_CFLAGS     += -fopenmp
	ifneq ($(G_DPCPP_EN),1)
		G_CXXFLAGS   += -fopenmp
		G_EXAMPLE_CFLAGS := $(G_EXE_CFLAGS) -DUSE_OMP -fopenmp
	else
		G_EXAMPLE_CFLAGS := $(G_EXE_CFLAGS)
	endif
ifeq ($(G_SYSTEM),macOS)
	G_LIB_LDFLAGS += -liomp5
endif
endif

#
# Export global vars
#
export G_SYSTEM
export G_MAKE_ROOT
export G_BUILD_ROOT
export G_ARCH
export G_CONF
export G_CONF_DIR
export G_USETBB
export G_USEOMP
export G_CFLAGS
export G_CXXFLAGS
export G_IPP_PATH_SUFFIX
export G_TBB_PATH_SUFFIX

export G_LIB_CFLAGS
export G_LIB_LDFLAGS
export G_EXE_CFLAGS
export G_EXE_LDFLAGS
export G_EXAMPLE_CFLAGS

export ARFLAGS
export G_DPCPP_EN
